## ------------------------------------------------------------------------- ##
##  Description: Generates the figures and tables for the manuscript
##    "Monitoring Global Digital Gender Inequality using the Online
##    Populations of Facebook and Google". 
##
## ------------------------------------------------------------------------- ##

# set appropriate working directory
wd <- ""
setwd(wd)

# read the dataset
datas <- read.csv("ADW_and_FB_digital_gender_gaps_dataset.csv")

# libraries
library(rworldmap)
library(ggplot2)
library(ggrepel)

# utility functions used in model fitting
source("functions.R")

## ----
## List of different variables in the analysis
## ----
#Facebook gender gap indicators, 18+ and different age groups
FBAgeGG <- c("FB_GG_age_18_plus", "FB_GG_age_15_19", "FB_GG_age_20_24",
             "FB_GG_age_25_29", "FB_GG_age_30_34", "FB_GG_age_35_39",
             "FB_GG_age_40_44", "FB_GG_age_45_49", "FB_GG_age_50_54",
             "FB_GG_age_55_59", "FB_GG_age_60_64", "FB_GG_age_65_plus",
             "FB_GG_age_18_23", "FB_GG_age_20_plus", "FB_GG_age_20_64",
             "FB_GG_age_21_plus", "FB_GG_age_25_plus", "FB_GG_age_25_49",
             "FB_GG_age_25_64", "FB_GG_age_50_plus","FB_GG_age_60_plus")

#Google AdWords gender gap indicators, 18+ and different age groups
ADW_GG <- c("ADW_GG_age_18_plus", "ADW_GG_age_25_plus", "ADW_GG_age_18_24",
            "ADW_GG_age_25_34", "ADW_GG_age_35_44", "ADW_GG_age_45_54",
            "ADW_GG_age_55_64", "ADW_GG_age_65_plus", "ADW_GG_age_25_54")

#Global gender gap report indicators (World Economic Forum Global Gender Gap Report)
GGIndicators <- c("GGG_score", "Econ_opp_GG_subindex", "Labr_force_GG",
                  "Wage_equal_GG", "Estim_income_GG", "Seniority_GG",
                  "Professionality_GG","Educ_attn_GG_subindex", "Literacy_GG",
                  "Enrol_prim_educ_GG", "Enrol_sec_educ_GG", "Enrol_ter_educ_GG")

#Development indicators from various sources (World Bank, UNDP)
devIndicators <- c("Int_pentr_IWS", "GDP_capita_PPP_2016", "HDI",
                   "Adult_literacy_HDI", "Educ_HDI", "Mean_yr_school_HDI",
                   "Sec_educ_HDI", "Unemployment_ratio", "Income_HDI")

#Hofstede's cultural dimensions
hofstede_dim <- c("pdi", "idv", "mas", "uai", "ltowvs", "ivr")

## ----
## Figure 1: Maps of the FB and ADW GGI 18+
## ----
## --------------------------------- Fig. 1: ADW/FB Maps ------------------------- #

## Map Plot parameters 
# The data to be plotted
df <- data.frame(ISO3Code = datas$ISO3Code,
                 Country = datas$Country,
                 FB_GG_age_18_plus = datas$FB_GG_age_18_plus,
                 ADW_GG_age_18_plus = datas$ADW_GG_age_18_plus)

# Intervals and their colours
cutVector <- c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1)
colourVector <- "topo"

## Plot the Maps 

# The FB GGI 18+
sPDF <- joinCountryData2Map(df, joinCode="ISO3", nameJoinColumn="ISO3Code",
                            nameCountryColumn="Country", 
                            suggestForFailedCodes=TRUE,
                            verbose=TRUE)
mapDevice()
mapParams <- mapCountryData(mapToPlot=sPDF, 
                            nameColumnToPlot="FB_GG_age_18_plus",
                            colourPalette=colourVector, 
                            catMethod=cutVector,
                            numCats=length(cutVector),
                            mapTitle="",
                            missingCountryCol="grey",
                            addLegend=FALSE)
do.call(addMapLegend,c(mapParams,legendLabels="all",legendIntervals="data"))

# The ADW GGI 18+
sPDF <- joinCountryData2Map(df, joinCode="ISO3", nameJoinColumn="ISO3Code",
                            nameCountryColumn="Country", 
                            suggestForFailedCodes=TRUE,
                            verbose=TRUE)
mapDevice()
mapParams <- mapCountryData(mapToPlot=sPDF, 
                            nameColumnToPlot="ADW_GG_age_18_plus",
                            colourPalette=colourVector, 
                            catMethod=cutVector,
                            numCats=length(cutVector),
                            mapTitle="",
                            missingCountryCol="grey",
                            addLegend=FALSE)
do.call(addMapLegend,c(mapParams,legendLabels="all",legendIntervals="data"))



## ----
## Figure 2: Scatter plot of FB GGI 18+ vs. ADW GGI 18+
## ----
dataToPlot <- data.frame(x = datas$FB_GG_age_18_plus,
                         y = datas$ADW_GG_age_18_plus,
                         Region = datas$Region_WB)
dataToPlot$Region <- as.character(dataToPlot$Region)
I <- dataToPlot$Region == "Latin America and the Caribbean"
dataToPlot$Region[I] <- "North/Latin America and\n the Caribbean"
I <- dataToPlot$Region == "North America"
dataToPlot$Region[I] <- "North/Latin America and\n the Caribbean"

## updated plot
ggplot(dataToPlot, aes(x = x, y = y, shape = Region)) + 
  theme_light() +
  theme(axis.text = element_text(size=17), axis.title=element_text(size=17,face="bold"), 
        legend.text=element_text(size=17), legend.title = element_text(size=17),
        legend.position =c(0.25,0.8), 
        legend.background = element_rect(fill="transparent")) +
  geom_abline(slope = 1, intercept = 0) + 
  scale_color_manual(values = c('red','green','blue','black','purple','brown','orange')) + 
  annotate(geom='text',label='Corr. 0.854',x=0.75,y=0.1,hjust=0,vjust=0,size=7) +
  geom_point(cex=3) + 
  coord_cartesian(xlim=c(0.1,1), ylim=c(0.1,1))  + 
  xlab("FB GGI (18+)") + ylab("ADW GGI (18+)") 



## ----
## Table 1: Summary statistics of Online/Offline variables
## ----
summary_df <- data.frame(variable = c("Internet_GG",
                                      "Using_copy_and_paste_tools_GG",
                                      "Using_basic_arithmetic_formula_in_spreadsheet_GG",
                                      "Writing_computer_program_using_programming_language_GG",
                                      "Transferring_files_between_a_computer_and_devices_GG",
                                      "FB_GG_age_18_plus",
                                      "FB_GG_age_20_64",
                                      "FB_GG_age_15_19",
                                      "FB_GG_age_18_23",
                                      "FB_GG_age_25_29",
                                      "FB_GG_age_25_49",
                                      "FB_GG_age_55_59",
                                      "FB_GG_age_50_54",
                                      "FB_GG_age_50_plus",
                                      "FB_GG_age_60_64",
                                      "ADW_GG_age_18_plus",
                                      "ADW_GG_age_25_plus",
                                      "ADW_GG_age_18_24",
                                      "HDI",
                                      "GDP_capita_PPP_2016",
                                      "Unemployment_ratio",
                                      "Mean_yr_school_HDI",
                                      "Seniority_GG",
                                      "Income_HDI",
                                      "Sec_educ_HDI"),
                         Q1 = NA,
                         Median = NA,
                         Mean = NA,
                         Q3 = NA,
                         Std_dev = NA,
                         stringsAsFactors = FALSE)

for (i in 1:nrow(summary_df)) {
  variable_name <- summary_df$variable[i]
  summ <- summary(datas[,variable_name])
  
  summary_df$Q1[i] <- round(as.numeric(summ["1st Qu."]),3)
  summary_df$Median[i] <- round(as.numeric(summ["Median"]),3)
  summary_df$Mean[i] <- round(as.numeric(summ["Mean"]),3)
  summary_df$Q3[i] <- round(as.numeric(summ["3rd Qu."]),3)
  summary_df$Std_dev[i] <- round(sqrt(var(datas[,variable_name],na.rm=TRUE)),3)
}

View(summary_df)

## ----
## Table 2: features correlation with ITU GGI
## ----
cor_table <- data.frame(variable = c("Internet_GG",
                                     "Int_pentr_IWS",
                                     "log_GDP_capita",
                                     "HDI",
                                     "Mean_yr_school_HDI",
                                     "FB_GG_age_18_plus",
                                     "FB_GG_age_20_64",
                                     "FB_GG_age_25_plus",
                                     "FB_GG_age_25_29",
                                     "ADW_GG_age_18_plus",
                                     "ADW_GG_age_25_plus"),
                        num_countries_in_data = NA,
                        cor_with_internet_GGI = NA,
                        stringsAsFactors = FALSE)
for (i in 1:nrow(cor_table)) {
  var_name <- cor_table$variable[i]
  cor_table$num_countries_in_data[i] <- sum(!is.na(datas[,var_name]))
  cor_table$cor_with_internet_GGI[i] <- cor(datas$Internet_GG,datas[,var_name],use="complete.obs")
}

View(cor_table)

## ----
## Table 3: Cor of DS GGI with FB and ADW GGI
## ----
ds_ggi_cor_table <- data.frame(DS_GGI = c("Copying_or_moving_file_folder_GG",
                                          "Using_copy_and_paste_tools_GG",
                                          "Sending_e.mails_with_attached_files_GG",
                                          "Using_basic_arithmetic_formula_in_spreadsheet_GG",
                                          "Connecting_installing_new_devices_GG",
                                          "Finding_downloading._installing_configuring_software_GG",
                                          "Creating_electronic_presentations_GG",
                                          "Transferring_files_between_a_computer_and_devices_GG",
                                          "Writing_computer_program_using_programming_language_GG"),
                               num_countries_with_FB = NA,
                               cor_with_FB_GG_age_18p = NA,
                               num_countries_with_ADW = NA,
                               cor_with_ADW_GG_age_18p = NA,
                               stringsAsFactors = FALSE)
for (i in 1:nrow(ds_ggi_cor_table)) {
  var_name <- ds_ggi_cor_table$DS_GGI[i]
  
  ds_ggi_cor_table$num_countries_with_FB[i] <- sum(!is.na(datas[,var_name]) & 
                                                     !is.na(datas$FB_GG_age_18_plus))
  ds_ggi_cor_table$cor_with_FB_GG_age_18p[i] <- cor(datas[,var_name], datas$FB_GG_age_18_plus,use="complete.obs")
  
  ds_ggi_cor_table$num_countries_with_ADW[i] <- sum(!is.na(datas[,var_name]) & 
                                                      !is.na(datas$ADW_GG_age_18_plus))
  ds_ggi_cor_table$cor_with_ADW_GG_age_18p[i] <- cor(datas[,var_name], datas$ADW_GG_age_18_plus,use="complete.obs")
}

View(ds_ggi_cor_table)


## ----
## Results Table 4
## ----

## Online Facebook Model
## ---------------------- ##
I <- !is.na(datas$Internet_GG) & !is.na(datas$FB_GG_age_18_plus)
Internet_on_modelData <- data.frame(Internet_GG = datas$Internet_GG[I], 
                                    FB_GG_age_18_plus = scale(datas$FB_GG_age_18_plus[I]))

Internet_online_model <- lm(Internet_GG ~ FB_GG_age_18_plus, Internet_on_modelData)
print("The Internet GG Online Model")
summary(Internet_online_model)

prd <- predict(Internet_online_model, Internet_on_modelData)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(Internet_on_modelData$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))


# Fit model with Leave-One-Out Cross Validation (LOOCV) so as to estimate a
# measure of out of sample predictive performance
I <- !is.na(datas$Internet_GG) & !is.na(datas$FB_GG_age_18_plus)
Internet_on_modelData <- data.frame(Internet_GG = datas$Internet_GG[I], 
                                    FB_GG_age_18_plus = datas$FB_GG_age_18_plus[I])
print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = Internet_on_modelData, response = "Internet_GG",
                     indicators = "")
pred[pred >1 ] <- 1

print("The SMAPE:")
error <- abs(Internet_on_modelData$Internet_GG - pred)
error <- error/((abs(Internet_on_modelData$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online Adwords model
## --------------------- ##
I <- !is.na(datas$Internet_GG) & !is.na(datas$ADW_GG_age_18_plus)
Internet_on_modelData2 <- data.frame(Internet_GG = datas$Internet_GG[I], 
                                     ADW_GG_age_18_plus = scale(datas$ADW_GG_age_18_plus[I]))

Internet_online_model2 <- lm(Internet_GG ~ ADW_GG_age_18_plus, Internet_on_modelData2)
print("The Internet GG Online Model")
summary(Internet_online_model2)

prd <- predict(Internet_online_model2)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(Internet_on_modelData2$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))


# Fit model with Leave-One-Out Cross Validation (LOOCV) so as to estimate a
# measure of out of sample predictive performance
I <- !is.na(datas$Internet_GG) & !is.na(datas$ADW_GG_age_18_plus)
Internet_on_modelData2 <- data.frame( Internet_GG = datas$Internet_GG[I], 
                                      ADW_GG_age_18_plus = datas$ADW_GG_age_18_plus[I])

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = Internet_on_modelData2, response = "Internet_GG",
                     indicators = "")
pred[pred > 1] <- 1

print("The SMAPE:")
error <- abs(Internet_on_modelData2$Internet_GG - pred)
error <- error/((abs(Internet_on_modelData2$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online Facebook + Adwords model
## ---------------------------------- ##
I <- !is.na(datas$Internet_GG) & 
  !is.na(datas$FB_GG_age_18_plus) & 
  !is.na(datas$ADW_GG_age_18_plus)
Internet_on_modelData3 <- data.frame(Internet_GG = datas$Internet_GG[I], 
                                     FB_GG_age_18_plus = scale(datas$FB_GG_age_18_plus[I]),
                                     ADW_GG_age_18_plus = scale(datas$ADW_GG_age_18_plus[I]))

Internet_online_model3 <- lm(Internet_GG ~ FB_GG_age_18_plus+ADW_GG_age_18_plus, Internet_on_modelData3)
print("The Internet GG Online Model")
summary(Internet_online_model3)

prd <- predict(Internet_online_model3)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(Internet_on_modelData3$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

# Fit model with Leave-One-Out Cross Validation (LOOCV) so as to estimate a
# measure of out of sample predictive performance
I <- !is.na(datas$Internet_GG) & !is.na(datas$FB_GG_age_18_plus) & !is.na(datas$ADW_GG_age_18_plus)
Internet_on_modelData3 <- data.frame(Internet_GG = datas$Internet_GG[I], 
                                     FB_GG_age_18_plus = scale(datas$FB_GG_age_18_plus[I]),
                                     ADW_GG_age_18_plus = scale(datas$ADW_GG_age_18_plus[I]))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = Internet_on_modelData3, response = "Internet_GG",
                     indicators = "")
pred[pred>1] <- 1
print("The SMAPE:")
error <- abs(Internet_on_modelData3$Internet_GG - pred)
error <- error/((abs(Internet_on_modelData3$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)




## Online-Offline Facebook Model
## ---------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim,
                      "FB_GG_age_18_plus")
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries)
selection <- 1:3

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online-Offline Adwords model
## --------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim,
                      "ADW_GG_age_18_plus")
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries)
selection <- 1:5

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online-Offline Facebook + Adwords model
## ---------------------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim,
                      "FB_GG_age_18_plus", "ADW_GG_age_18_plus")
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries)
selection <- 1:5

modelVars <- c(colnames(finModel_variables)[selection],"FB_GG_age_18_plus")
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)

## Offline model
## --------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim)
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries)
selection <- 1:4

modelVars <- c(colnames(finModel_variables)[selection])
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## ----
## Results Table 5
## ----

## Online Facebook Model
## ---------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(FBAgeGG)
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries, check_sig = FALSE)
selection <- 1:4

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online Adwords model
## --------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(ADW_GG)
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries, check_sig = FALSE)
selection <- 1:2

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online Facebook + Adwords model
## ---------------------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG, datas$ADW_GG_age_18_24)
names(baseModel) <- c("Internet_GG","ADW_GG_age_18_24")
varyName <- "Internet_GG"

candidVars_names <- c(FBAgeGG, setdiff(ADW_GG,"ADW_GG_age_18_24"))
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries, check_sig = FALSE)
selection <- 1:5

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)



## Online-Offline Facebook Model
## ---------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim,
                      FBAgeGG)
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries, check_sig = FALSE)
selection <- 1:5

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online-Offline Adwords model
## --------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim,
                      ADW_GG)
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries, check_sig = FALSE)
selection <- 1:3

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## Online-Offline Facebook + Adwords model
## ---------------------------------- ##
# setup
baseModel <- data.frame(datas$Internet_GG)
names(baseModel) <- c("Internet_GG")
varyName <- "Internet_GG"

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim,
                      FBAgeGG, ADW_GG)
candidVars <- datas[,candidVars_names]
minCountries <- 70

# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries, check_sig = FALSE)
selection <- 1:5

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(Internet_GG ~ ., modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled$Internet_GG - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data$Internet_GG - pred)
error <- error/((abs(model_data$Internet_GG)+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)


## ----
## Results Table 6
## ----

## Online-Offline Facebook + Adwords model
## ---------------------------------- ##

## Select the Skills Gender Gap to model
DS_GG <- "Using_copy_and_paste_tools_GG"
DS_GG <- "Using_basic_arithmetic_formula_in_spreadsheet_GG"
DS_GG <- "Writing_computer_program_using_programming_language_GG"
DS_GG <- "Transferring_files_between_a_computer_and_devices_GG"


# setup
baseModel <- data.frame(datas[,DS_GG])
names(baseModel) <- c(DS_GG)
varyName <- DS_GG

candidVars_names <- c(GGIndicators, devIndicators, hofstede_dim,
                      FBAgeGG, ADW_GG)
candidVars <- datas[,candidVars_names]
minCountries <- sum(!is.na(datas[,DS_GG]))


##
# Fit the model
finModel_variables <- build_model_with_greedy_stepwise_forward(
  baseModel, varyName, candidVars, minCountries, check_sig = FALSE, 
  max_vars = 2)
selection <- 1:3

modelVars <- colnames(finModel_variables)[selection]
I <- rowSums(!is.na(datas[,modelVars])) == length(modelVars) 
model_data <- datas[I,modelVars]

# model summary with standardized variables
modData_scaled <- data.frame(scale(datas[I,modelVars]))
modData_scaled[,varyName] <- datas[I,varyName]

Internet_model <- lm(as.formula(paste(DS_GG," ~ .")), modData_scaled)
summary(Internet_model)

# Model error evaluation
prd <- predict(Internet_model, modData_scaled)
prd[prd > 1] <- 1
Mean_abs_error <- mean(abs(modData_scaled[,DS_GG] - prd))
print(paste("Model Mean Absolute Error:",Mean_abs_error))

print("Results of LOOCV for testing predictive power")
pred <- predictLOOCV(dataset = model_data, response = varyName,
                     indicators = "")
pred[pred > 1] <- 1
print("The SMAPE:")
error <- abs(model_data[,DS_GG] - pred)
error <- error/((abs(model_data[,DS_GG])+abs(pred))/2)
error <- sum(error)/length(pred)
print(error)

## ----
## Figure 3: Maps for DS GGI - using copy and paste tools
## Figure 4: Scatter plot of predicted vs. ground truth
## ----

## ---------------------------------  Figure 3 Maps ------------------------- #
DS_GG_models <- lm(Using_copy_and_paste_tools_GG ~ FB_GG_age_18_23 + Income_HDI,datas)
DS_GG_pred <- predict(DS_GG_models,datas)
DS_GG_pred[!is.na(DS_GG_pred) & DS_GG_pred >1] <- 1

## Map Plot parameters 
# The data to be plotted
df <- data.frame(ISO3Code = datas$ISO3Code,
                 Country = datas$Country,
                 Using_copy_and_paste_tools_GG = datas$Using_copy_and_paste_tools_GG,
                 prediction = DS_GG_pred)

# Intervals and their colours
cutVector <- c(0.3,0.5,0.6,0.7,0.8,0.9,1)
colourVector <- c("#FFE0B3FF", "#FFFF00FF", "#E6FF00FF", "#00FF4DFF",
                  "#00E5FFFF", "#4C00FFFF")

## Plot the Maps 

# The Ground Truth 
sPDF <- joinCountryData2Map(df, joinCode="ISO3", nameJoinColumn="ISO3Code",
                            nameCountryColumn="Country", 
                            suggestForFailedCodes=TRUE,
                            verbose=TRUE)
mapDevice()
mapParams <- mapCountryData(mapToPlot=sPDF, 
                            nameColumnToPlot="Using_copy_and_paste_tools_GG",
                            colourPalette=colourVector, 
                            catMethod=cutVector,
                            numCats=length(cutVector),
                            mapTitle="",
                            missingCountryCol="grey",
                            addLegend=FALSE)
do.call(addMapLegend,c(mapParams,legendLabels="all",legendIntervals="data"))

# The prediction by the Model
sPDF <- joinCountryData2Map(df, joinCode="ISO3", nameJoinColumn="ISO3Code",
                            nameCountryColumn="Country", 
                            suggestForFailedCodes=TRUE,
                            verbose=TRUE)
mapDevice()
mapParams <- mapCountryData(mapToPlot=sPDF, 
                            nameColumnToPlot="prediction",
                            colourPalette=colourVector, 
                            catMethod=cutVector,
                            numCats=length(cutVector),
                            mapTitle="",
                            missingCountryCol="grey",
                            addLegend=FALSE)
do.call(addMapLegend,c(mapParams,legendLabels="all",legendIntervals="data"))


# ---------------------- Figure 4: scatter plot ----------------------------- #
ggplot(df, aes(x = Using_copy_and_paste_tools_GG, 
               y = prediction)) + 
  theme_light() +
  theme(axis.text = element_text(size=17), axis.title=element_text(size=17,face="bold"), 
        legend.text=element_text(size=17), legend.title = element_text(size=17),
        legend.position =c(0.25,0.8), 
        legend.background = element_rect(fill="transparent")) +
  geom_abline(slope = 1, intercept = 0) + 
  geom_point(cex=3) + geom_text_repel(aes(label = Country)) +
  xlab("DS GGI: copy & paste tools") + ylab("Predicted") 




## ----
##  Appendix Tables 7 and 8
## ----

## -- Run the relevant lines of code to 
## compute either Table 7 or Table 8
#*# Table 7 Models
response <- "Internet_GG_uncapped_log"
model_vars <- list()
model_vars[["Online_FB"]] <- c("FB_GG_age_18_plus_uncapped_log")
model_vars[["Online_ADW"]] <- c("ADW_GG_age_18_plus_uncapped_log")
model_vars[["Online_FB_ADW"]] <- c("FB_GG_age_18_plus_uncapped_log","ADW_GG_age_18_plus_uncapped_log")
model_vars[["Online_Offline_FB"]] <- c("HDI","FB_GG_age_18_plus_uncapped_log")
model_vars[["Online_Offline_ADW"]] <- c("GDP_capita_PPP_2016","HDI",
                                        "Mean_yr_school_HDI","ADW_GG_age_18_plus_uncapped_log")
model_vars[["Online_Offline_FB_ADW"]] <- c("GDP_capita_PPP_2016","HDI",
                                           "Mean_yr_school_HDI",
                                           "FB_GG_age_18_plus_uncapped_log","ADW_GG_age_18_plus_uncapped_log")
model_vars[["Offline"]] <- c("GDP_capita_PPP_2016","HDI",
                             "Unemployment_ratio")


#*# Table 8 Models
response <- "Internet_GG_uncapped_log"
model_vars <- list()
model_vars[["Online_FB"]] <- c("FB_GG_age_25_49_uncapped_log","FB_GG_age_55_59_uncapped_log","FB_GG_age_60_64_uncapped_log")
model_vars[["Online_ADW"]] <- c("ADW_GG_age_18_24_uncapped_log")
model_vars[["Online_FB_ADW"]] <- c("FB_GG_age_18_plus_uncapped_log","FB_GG_age_50_54_uncapped_log",
                                   "FB_GG_age_60_64_uncapped_log","ADW_GG_age_18_24_uncapped_log")
model_vars[["Online_Offline_FB"]] <- c("HDI","FB_GG_age_20_64_uncapped_log","FB_GG_age_25_29_uncapped_log",
                                       "FB_GG_age_60_64_uncapped_log")
model_vars[["Online_Offline_ADW"]] <- c("HDI","ADW_GG_age_18_24_uncapped_log")
model_vars[["Online_Offline_FB_ADW"]] <- c("HDI","FB_GG_age_50_plus_uncapped_log",
                                           "FB_GG_age_60_64_uncapped_log",
                                           "ADW_GG_age_18_24_uncapped_log")
model_vars[["Offline"]] <- c("GDP_capita_PPP_2016","HDI",
                             "Unemployment_ratio")

## -- Run below to generate the appendix table chosen above
full_vars <- c()
for (mdl in names(model_vars)) {
  full_vars <- c(full_vars, model_vars[[mdl]])
}
full_vars <- full_vars[!duplicated(full_vars)]
err_metrics <- c("Adj_Rsqr","MAE","SMAPE","Fstat","Df","N","Num_countries_pred")

model_results <- data.frame(var = c("(Intercept)",full_vars,err_metrics), 
                            row.names = c("(Intercept)",full_vars,err_metrics),
                            stringsAsFactors = FALSE)

for (mdl in names(model_vars)) {
  cat("Fitting Model:",mdl,"                         \r")
  mdl_vars <- model_vars[[mdl]]
  model_results[mdl] <- NA
  
  
  # pick subset of data
  I <- !is.na(datas[,response]) 
  for (varn in mdl_vars) {
    I <- I & !is.na(datas[,varn])
  }
  datasub <- datas[I,c(response,mdl_vars)]
  
  
  # model summary with standardized variables
  modData_scaled <- datas[I,c(response,mdl_vars)]
  for (vv in mdl_vars) {
    modData_scaled[,vv] <- scale(modData_scaled[,vv])[,1]
  }
  data_model <- lm(as.formula(paste(response,"~.")), modData_scaled)
  
  mdv <- names(summary(data_model)$coefficients[,1])
  mdd <- summary(data_model)$coefficients[,1]
  model_results[mdv,mdl] <- mdd
  
  # record error metrics
  model_results["Adj_Rsqr",mdl] <- summary(data_model)$adj.r.squared
  
  model_results["Fstat",mdl] <- summary(data_model)$fstatistic[1]
  model_results["Df",mdl] <- summary(data_model)$df[2]
  model_results["N",mdl] <- nrow(modData_scaled)
  
  # MAE
  data_model <- lm(as.formula(paste(response,"~.")), datasub)
  # invert back to 0-1 scale and compute error
  gt <- exp(datasub[,response])
  gt[gt>1] <- 1
  prd <- predict(data_model, datasub)
  prd <- exp(prd)
  prd[prd>1] <- 1 
  Mean_abs_error <- mean(abs(gt - prd))
  
  model_results["MAE",mdl] <- Mean_abs_error
  
  # SMAPE error metric
  pred <- predictLOOCV(dataset = datasub, response = response,
                       indicators = "")
  gt <- exp(datasub[,response])
  gt[gt >1] <- 1
  pred <- exp(pred)
  pred[pred>1] <- 1
  
  error <- abs(gt - pred)
  error <- error/((abs(gt)+abs(pred))/2)
  error <- sum(error)/length(pred)
  
  model_results["SMAPE",mdl] <- error
  
  # computer country coverage
  I <- rep(TRUE, nrow(datas))
  for (varn in model_vars[[mdl]]) {
    I <- I & !is.na(datas[,varn])
  }
  num_preds <- sum(I)
  model_results["Num_countries_pred",mdl] <- num_preds
}

View(model_results)


## ----
## Appendix Tables 9 and 10: Likelihood-ratio/Vuong tests for models
## ----

## a list of the variables in the various models
response <- "Internet_GG"
model_vars <- list()
model_vars[["Tab4_FB"]] <- c("FB_GG_age_18_plus")
model_vars[["Tab4_ADW"]] <- c("ADW_GG_age_18_plus")
model_vars[["Tab4_FB_ADW"]] <- c("FB_GG_age_18_plus","ADW_GG_age_18_plus")

model_vars[["Tab4_Off_FB"]] <- c("HDI","FB_GG_age_18_plus")
model_vars[["Tab4_Off_ADW"]] <- c("GDP_capita_PPP_2016","HDI",
                                  "Mean_yr_school_HDI","ADW_GG_age_18_plus")
model_vars[["Tab4_Off_FB_ADW"]] <- c("GDP_capita_PPP_2016","HDI",
                                     "Mean_yr_school_HDI",
                                     "FB_GG_age_18_plus","ADW_GG_age_18_plus")
model_vars[["Tab5_FB"]] <- c("FB_GG_age_25_49","FB_GG_age_55_59","FB_GG_age_60_64")
model_vars[["Tab5_ADW"]] <- c("ADW_GG_age_18_24")
model_vars[["Tab5_FB_ADW"]] <- c("FB_GG_age_18_plus","FB_GG_age_50_54",
                                 "FB_GG_age_60_64","ADW_GG_age_18_24")
model_vars[["Tab5_Off_FB"]] <- c("HDI","FB_GG_age_20_64","FB_GG_age_25_29","FB_GG_age_60_64")
model_vars[["Tab5_Off_ADW"]] <- c("HDI","ADW_GG_age_18_24")
model_vars[["Tab5_Off_FB_ADW"]] <- c("HDI","FB_GG_age_50_plus",
                                     "FB_GG_age_60_64","ADW_GG_age_18_24")

## Perform likelihood ratio/Vuong tests 
## comparing different pairs of models
tests_df <- data.frame(model_1 = c("Tab4_FB","Tab4_ADW","Tab5_FB","Tab5_ADW","Tab4_FB_ADW",
                                   "Tab4_Off_FB","Tab4_Off_ADW","Tab5_Off_FB","Tab5_Off_ADW","Tab4_Off_FB_ADW"),
                       model_1_adjRsqr = NA,
                       model_2 = c(rep("Tab4_FB_ADW",2),rep("Tab5_FB_ADW",3),
                                   rep("Tab4_Off_FB_ADW",2),rep("Tab5_Off_FB_ADW",3)),
                       model_2_adjRsqr = NA,
                       N = NA,
                       models_nested = NA,
                       test_statistic = NA,
                       pvalue = NA, stringsAsFactors = FALSE)

for (i in 1:nrow(tests_df)) {
  mod1_vars <- model_vars[[tests_df$model_1[i]]]
  mod2_vars <- model_vars[[tests_df$model_2[i]]]
  
  vrs <- c(mod1_vars,mod2_vars)
  vrs <- vrs[!duplicated(vrs)]
  
  # pick common subset of the data
  I <- !is.na(datas[,response])
  for (vv in vrs) { I <- I & !is.na(datas[,vv])}
  datasub <- datas[I,]
  
  # fit both models on the common subset of the data
  lmformula1 <- paste(response,"~",mod1_vars[1])
  lmformula2 <- paste(response,"~",mod2_vars[1])
  for (vv in mod1_vars[-1]) {
    lmformula1 <- paste(lmformula1,"+",vv)
  }
  for (vv in mod2_vars[-1]) {
    lmformula2 <- paste(lmformula2,"+",vv)
  }
  
  mod1 <- lm(lmformula1, datasub)
  mod2 <- lm(lmformula2, datasub)
  
  tests_df$model_1_adjRsqr[i] <- summary(mod1)$adj.r.squared
  tests_df$model_2_adjRsqr[i] <- summary(mod2)$adj.r.squared
  tests_df$N[i] <- nrow(datasub)
  
  # check if the models are nested
  tests_df$models_nested[i] <- sum(is.element(mod1_vars, mod2_vars)) == length(mod1_vars)
  
  # perform the test
  if (tests_df$models_nested[i]) {
    # perform the likelihood ratio test
    lt <- lrtest(mod1,mod2)
    
    tests_df$test_statistic[i] <- lt$Chisq[2]
    tests_df$pvalue[i] <- lt$`Pr(>Chisq)`[2]
  } else {
    # preform the Vuong non-nested test
    vt <- vuongtest(mod1,mod2, nested = FALSE)
    
    tests_df$test_statistic[i] <- vt$LRTstat
    tests_df$pvalue[i] <- vt$p_LRT$B
  }
}

View(tests_df)