## ------------------------------------------------------------------------- ##
##  Description: Some utility functions for model fitting.
## ------------------------------------------------------------------------- ##

##

build_model_with_greedy_stepwise_forward <-
  function(baseModel, varyName, candidVars, minCountries = 0, check_sig = TRUE, max_vars = 6) {
    # variables in the best model so far including the response variable
    inModel <- baseModel
    # rename response var for now
    names(inModel)[names(inModel)==varyName] <- "response_var" 
    
    # variables to consider including in the model
    candidVarsNames <- names(candidVars)
    
    cnt <- 0
    
    while (length(candidVarsNames) > 0 & (ncol(inModel) < (max_vars+1))) {
      # as long as there is a variable to try
      cnt <- cnt + 1
      
      # what is the best adjusted R-squared achieved so far
      bestAdjRsqr <- summary(lm(response_var ~ .,inModel))$adj.r.squared
      idx = 0 # index of variable to enter the model next
      bestCountryCnt <- 0
      
      for (i in 1:length(candidVarsNames)) {
        # for each candidate variable check the adj R squared 
        # if it is added to the model and keep track of the one 
        # that gives the highest adjusted r-squared while leaving
        # all model variables significant at the 5% level
        tempModel <- data.frame(inModel,candidVars[,i])
        names(tempModel)[length(names(tempModel))] <- candidVarsNames[i] 
        
        model <- lm(response_var ~., tempModel)
        adjRsqr <- summary(model)$adj.r.squared
        #rw <- names(summary(model)$coefficients[,4]) == candidVarsNames[i] 
        l <- length(summary(model)$coefficients[,4])
        variablePval <- summary(model)$coefficients[,4][2:l]
        
        # no. of estimated coefficients excluding intercept
        ncoeff <- length(names(tempModel)) - 1 
        
        # How many countries is this model based on?
        countryCnt <- rep(1,length(tempModel[,1]))
        for (j in 1:length(names(tempModel))) {
          countryCnt <- countryCnt & !is.na(tempModel[,j])
        }
        countryCnt <- sum(countryCnt)
        
        # keep track of the variable that satisfies the
        # conditions. 
        cond <- adjRsqr > bestAdjRsqr & countryCnt >= minCountries
        if (check_sig) { cond <- cond & sum(variablePval < 0.05) == ncoeff}
        
        if (cond) {
          bestAdjRsqr <- adjRsqr
          idx = i
          bestCountryCnt <- countryCnt
        }
      }
      # if there is no variable to be added, stop
      if (idx == 0) {break}
      
      # else, update the current model by incorporating the 
      # next variable into the model
      inModel[,candidVarsNames[idx]] <- candidVars[,idx]
      
      candidVars <- candidVars[,-idx]
      candidVarsNames <- names(candidVars)
      
      # print the results of this iteration
      print(paste("round",cnt,"results"))
      print(paste("Model fitted on",bestCountryCnt,"Countries"))
      print("------------------------------------------------------")
      print(summary(lm(response_var ~., inModel)))
      print("")
      print("### =============================================================== ###")
    }
    
    # Revert the name of the explanatory 
    # variable back to what it was
    names(inModel)[names(inModel)=="response_var"] <- varyName 
    
    return(inModel)
  }


##

predictLOOCV <- function(dataset, response, indicators) {
  
  # for convenience changes the name of the response variable
  colnames(dataset)[colnames(dataset) == response] <- "response_var"
  
  # go through the dataset, leaving out one observations at a time
  # as the test set
  predictions <- c() # vector to record the predictions
  n <- nrow(dataset)
  for (i in 1:n) {
    # leave out row i as test data
    train.data <- dataset[-i,]
    test.data <- dataset[i,]
    
    # scale all the variables
    # columns to be scaled (only the inputs that are not indicator variables)
    Idx <- !is.element(colnames(train.data), c("response_var",indicators)) 
    train.scaled <- scale(train.data[,Idx])
    train.data[,Idx] <- train.scaled[,]
    
    scale.center <- attr(train.scaled, "scaled:center") # centering value
    scale.scale <- attr(train.scaled, "scaled:scale")   # std. dev. value
    
    test.data[,Idx] <- scale(test.data[,Idx], 
                             center = scale.center, scale = scale.scale)[,]
    
    # fit the model to the training data
    model <- lm(response_var ~ . , train.data)
    
    # predict on the test data
    pred <- predict(model, test.data)
    predictions <- c(predictions, pred)
    #cat("how many predictions: ",length(pred)," -- ")
  }
  return(predictions)
}
